%% Experiment 3
% %% Uses structure objects

% Compressed Sensing in On-Grid MIMO Radar
% Test Simulations, SNR = 15, 30, Inf dB
% Random array
% L1 Square Non Negative Regularization
% ROC Curves for L1SQNN as sparsity varies
% Size of the problem is 500*1000
% Sparsity 1 to 15

% Analyze different threshold levels [0.0,0.0001,0.001,0.01,0.1,0.2,0.3,0.4,0.5,0.6,0.7,.8,0.9,.99,.999]

% Michael F Minner

%% Clear
clear all;

%% Simulation information

% For each sparsity level, generate N_S signals, N_M matrices
Sparsity_Levels=[1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,20];

N_S=40;N_A=25;N_ss=size(Sparsity_Levels,2);  % Number of signals, random arrays, sparsity levels
N_Sim=N_S*N_A;                               % Number of simulations for each sparsity level

threshold=[0.0,0.0001,0.001,0.01,0.1,0.2,0.3,0.4,0.5,0.6,0.7,.8,0.9,.99,.999];                     % Consider different threshold levels for detection
N_thresh=size(threshold,2);                  % Number of thresholds considered

SNR=[Inf,30,15];                                      % Signal to Noise Ratio in dB
N_snr=size(SNR,2);                           % Number of Signal to Noise Ratios


Par_L1SQ=[24,15,5];                          % Consider different parameter values for lambda
N_par=size(Par_L1SQ,2);                      % Number of Parameter values

% Average of the probabilities of detection
% and false alarms for each threshold level
% Average relative error with thresholding
% Average relative error without thresholding
% Average time to evaluate OMP
% Average number of iterations for OMP

L1SQNNR.AvgProbD=zeros(N_ss,N_thresh, N_snr);       % Random Array
L1SQNNR.AvgProbFA=zeros(N_ss,N_thresh, N_snr);
L1SQNNR.AvgRelErr=zeros(N_ss,N_thresh, N_snr);
L1SQNNR.AvgRelErrS=zeros(N_ss,N_thresh, N_snr);
L1SQNNR.AvgTime=zeros(N_ss, 1, N_snr);
L1SQNNR.AvgNbrItr=zeros(N_ss, 1, N_snr);

L1SQNNR_ProbD=zeros(N_Sim,N_thresh, N_snr);
L1SQNNR_ProbFA=zeros(N_Sim,N_thresh, N_snr);
L1SQNNR_RelErr=zeros(N_Sim,N_thresh, N_snr);
L1SQNNR_RelErrS=zeros(N_Sim,N_thresh, N_snr);
L1SQNNR_Time=zeros(N_Sim, 1, N_snr);
L1SQNNR_NbrItr=zeros(N_Sim, 1, N_snr);


% each row is a separate simulation,
% each column is a different threshold
% each set of rows and columns corresponds to a different SNR level.

fname=(['Experiment_3_SNR_', num2str(SNR(1)),'_ThreshNbr_', num2str(N_snr),'.mat']);

%% Initialization

M=5;         % # of Transmitters
N=5;         % # of Receivers.
Q=20;         % # of samples

U=10;         % # of Azimuth Bins
V=10;         % # of Range Bins
W=10;         % # of Doppler Bins

K=U*V*W;      % total # of Bins

c=3e8;             % speed of light

B=15e6;            % Bandwidth of each transmitted signal 15MHz
T=2e-6;            % Waveform pulse duration
f_c=1e10;          % Carrier frequency of the first transmitter 10GHz
lambda=c/f_c;      % Reference carrier wavelength, lambda = c/f_c
alpha=B/T;         % Chirp rate, alpha=B/T

Range_U=[-.1,.1];                 % range of angles (-.1 to .1) in radians
Range_V=[990,1010];               % range of distances in meters (980 to 1020 meters)
Range_W=[0,30];                   % range of speeds (0 to 30 meters per second)

Range_tau=2.*Range_V./c;          % range of time delays
Range_upsilon=2/lambda.*Range_W;  % range of Doppler shifts

Delta_U=(Range_U(2)-Range_U(1))/U;      % step size in each domain
Delta_V=(Range_V(2)-Range_V(1))/V;
Delta_W=(Range_W(2)-Range_W(1))/W;

Delta_tau=(Range_tau(2)-Range_tau(1))/V;
Delta_upsilon=(Range_upsilon(2)-Range_upsilon(1))/W;

Values_U=Range_U(1):Delta_U:Range_U(2)-Delta_U; % values for each bin
Values_V=Range_V(1):Delta_V:Range_V(2)-Delta_V; % remove additional final bin
Values_W=Range_W(1):Delta_W:Range_W(2)-Delta_W;

Values_tau=Range_tau(1):Delta_tau:Range_tau(2)-Delta_tau;
Values_upsilon=Range_upsilon(1):Delta_upsilon:Range_upsilon(2)-Delta_upsilon;

% Sample in time only for targets in the scene...
T_d=Range_tau(2)-Range_tau(1);  % Sample time duration
Delta_Q=T_d/Q;                  % Sample times
Values_T_d=Range_tau(1):Delta_Q:Range_tau(2)-Delta_Q;

%% Simulations

timeStart=tic;
index1=1; % Index for the total number of simulations performed

index4=1; % Index for sparsity

for ss=Sparsity_Levels   % Vary the sparsity level
    
    index2=1; % Index for number of simulations per sparsity
    
    % Calculations for measurement matrices
    [~, weight,A_F, A_G, A_H] = LinearChirp( M,Q,V,W,alpha,T,f_c,Values_tau, Values_upsilon, Values_T_d );
    
    for ii=1:N_A          % The number of matrices to generate
        
        % Random Antenna Locations
        [ p1,q1 ] = AntennaPositionRandom(M,N);
        
        
        % Build Antenna Arrays
        [A_T1,A_R1] = AntennaArray( p1,q1,lambda, M,N,U,Values_U);
        
        % Measurement Matrices
        [ A1 ] = MeasurementMatrixVectorized(A_R1, A_T1, A_F, A_G, A_H, weight,M,N,Q,U,V,W,K);
        
        
        
        for kk=1:N_S  % The number of signals to generate
            
            %Mag=2;            % Magnitude of target reflectivities
            [signal, grid, supp] = TargetScene(ss,U,V,W,K);
            
            % All ones on the support
            signal(supp)=ones(ss,1);
            
            % Take Measurements
            y1=A1*signal;
            
            % Complex separation
            x=[real(signal);imag(signal)];
            B1=[real(A1),-imag(A1);imag(A1),real(A1)];
            z1=B1*x;
            
            
            % Signal Power
            PowerSignal=norm(signal(supp)).^2;
            snr=SNR;
            index5=1;
            for snr=SNR
                
                if snr==Inf
                    
                    tStartL1SQNNR=tic;
                    display(['current SNR: ' num2str(snr)])
                    [xL1SQNNR,~,~,NbrItrL1SQNNR] = l1sqnnreg(z1,B1,Par_L1SQ(index5));
                    L1SQNNR_Time(index2,1,index5)=toc(tStartL1SQNNR);
                    L1SQNNR_NbrItr(index2,1,index5)=NbrItrL1SQNNR;
                    xL1SQNNR=xL1SQNNR(1:K)+1i.*xL1SQNNR(K+1,end);
                    %L1SQNNR_RelErrNT(index2,1,index5)=norm(xL1SQNNR-signal)/sqrt(PowerSignal);
                    xL1SQNNR_Abs=abs(xL1SQNNR);
                    
                else
                    % Generate Noise
                    
                    NoiseStdDev=sqrt(PowerSignal/(2*M*N*Q*power(10,snr/10)));                % Standard deviation of the noise
                    noise=NoiseStdDev.*randn(M*N*Q,1)+NoiseStdDev*1i.*randn(M*N*Q,1);        % Complex Gaussian noise
                    
                    %noisey measurments
                    y1N=y1+noise;
                    z1N=[real(y1N);imag(y1N)];
                    
                    % Recovery
                    
                    % L1SQNN
                    tStartL1SQNNR=tic;
                    display(['current SNR: ' num2str(snr)])
                    [xL1SQNNR,~,~,NbrItrL1SQNNR] = l1sqnnreg(z1N,B1,Par_L1SQ(index5));
                    L1SQNNR_Time(index2,1,index5)=toc(tStartL1SQNNR);
                    L1SQNNR_NbrItr(index2,1,index5)=NbrItrL1SQNNR;
                    xL1SQNNR=xL1SQNNR(1:K)+1i.*xL1SQNNR(K+1,end);
                    %L1SQNNR_RelErrNT(index2,1,index5)=norm(xL1SQNNR-signal)/sqrt(PowerSignal);
                    
                    xL1SQNNR_Abs=abs(xL1SQNNR);
                end
                
                index3=1;
                for t=threshold
                    
                    Removed_L1SQNNR=find(xL1SQNNR_Abs<=t);
                    xL1SQNNR(Removed_L1SQNNR)=0;
                    xL1SQNNR_Abs(Removed_L1SQNNR)=0;
                    SuppL1SQNNR=find(xL1SQNNR_Abs>t);
                    L1SQNNR_RelErr(index2,index3,index5)=norm(xL1SQNNR-signal)/sqrt(PowerSignal);
                    L1SQNNR_ProbD(index2,index3,index5)=sum(xL1SQNNR_Abs(supp,1)>=t)/ss;
                    L1SQNNR_ProbFA(index2,index3,index5)=sum(xL1SQNNR_Abs(setdiff(SuppL1SQNNR,supp),1)>=t)/(K-ss);
                    L1SQNNR_RelErrS(index2,index3,index5)=norm(xL1SQNNR(supp)-signal(supp))/sqrt(PowerSignal);
                    index3=index3+1;
                end
                
                index5=index5+1;
                
                
            end
            display(['current sparsity: ' num2str(ss) ' current simulation: ' num2str(index2)])
            index2=index2+1;
            index1=index1+1;
        end
    end
    
    
    L1SQNNR.AvgProbD(index4,:,:)=mean(L1SQNNR_ProbD,1);
    L1SQNNR.AvgProbFA(index4,:,:)=mean(L1SQNNR_ProbFA,1);
    L1SQNNR.AvgRelErr(index4,:,:)=mean(L1SQNNR_RelErr,1);
    %L1SQNNR.AvgRelErrNT(index4,1,:)=mean(L1SQNNR_RelErrNT,1);
    L1SQNNR.AvgRelErrS(index4,:,:)=mean(L1SQNNR_RelErrS,1);
    L1SQNNR.AvgTime(index4,1,:)=mean(L1SQNNR_Time,1);
    L1SQNNR.AvgNbrItr(index4,1,:)=mean(L1SQNNR_NbrItr,1);
    
    index4=index4+1;
    
    save(fname, 'L1SQNNR' ,'Sparsity_Levels','SNR', 'Par_L1SQ', 'threshold' ,'index1', 'index4', 'M', 'N', 'Q', 'U', 'V', 'W', 'K', 'Range_U', 'Range_V', 'Range_W');
    
end

timeStop=toc(timeStart)

